/*
 * Copyright (c) 2002 University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of SPAK.
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation is hereby granted without fee, provided that the
 * above copyright notice and this permission/disclaimer notice is
 * retained in all copies or modified versions, and that both notices
 * appear in supporting documentation.  THE COPYRIGHT HOLDERS PROVIDE
 * THIS SOFTWARE "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE COPYRIGHT
 * HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER
 * RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu
 * any improvements that they make and grant redistribution rights to the
 * University of Utah.
 *
 * Author: John Regehr (regehr@cs.utah.edu)
 */

#include "spak_public.h"

int main (void) 
{
  struct task_set *ts;
  int feas;

  max_resp = 100000;
  ANNEAL_MAX = 1000;
  INIT_TEMP = 0.03;
  TEMP_SCALE = 0.99;

  ts = create_task_set (3, 0, 0, 3,
			"preemption_thresh_rtcsa_table1", 
			1000, 0, 0, 0,
			"Wang00_fixed");
  //                    C    T    D    J    B
  new_simple_task (ts, 20,  70,  50,   0,   0, "t1");
  new_simple_task (ts, 20,  80,  80,   0,   0, "t2");
  new_simple_task (ts, 35, 200, 100,   0,   0, "t3");

  set_priorities (ts, INORDER);

  make_all_preemptible (ts);
  assign_optimal_pri (ts);
  feas = feasible (ts, FALSE);
  if (feas == num_tasks (ts)) {
    printf ("found a fully preemptive schedule:\n");
    print_task_set (ts);
  } else {
    printf ("optimal preemptive algorithm did not find a schedule\n");
  }
  
  printf ("\n");

  make_all_nonpreemptible (ts);
  assign_optimal_pri (ts);
  feas = feasible (ts, FALSE);
  if (feas == num_tasks (ts)) {
    printf ("found a fully non-preemptive schedule:\n");
    print_task_set (ts);
  } else {
    printf ("optimal non-preemptive algorithm did not find a schedule\n");
  }

  printf ("\n");

  anneal_priorities_and_thresholds (&ts, TRUE);
  feas = feasible (ts, FALSE);
  if (feas == num_tasks (ts)) {
    printf ("found a schedule using preemption thresholds:\n");
    print_task_set (ts);
  } else {
    printf ("heuristic preemption threshold algorithm did not find a schedule\n");
  }

  return 0;
}
