/*
 * Copyright (c) 2002 University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of SPAK.
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation is hereby granted without fee, provided that the
 * above copyright notice and this permission/disclaimer notice is
 * retained in all copies or modified versions, and that both notices
 * appear in supporting documentation.  THE COPYRIGHT HOLDERS PROVIDE
 * THIS SOFTWARE "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE COPYRIGHT
 * HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER
 * RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu
 * any improvements that they make and grant redistribution rights to the
 * University of Utah.
 *
 * Author: John Regehr (regehr@cs.utah.edu)
 */

#include "spak_public.h"

int main (void) 
{
  struct task_set *ts;
  int feas, i;
  
  max_resp = 1000000;

  ts = create_task_set (18, 10, 500, 0,
			"audsley93_t3", 
			1000, 0, 0, 0,
			"Tindell92_restricted");

  //                       C       T      D  J  B  name
  new_simple_task (ts,  3500, 200000,  5000, 0, 0, "t1");
  new_simple_task (ts,  2000,  25000, 25000, 0, 0, "t2");
  new_simple_task (ts,  5000,  25000, 25000, 0, 0, "t3");
  new_simple_task (ts,  1000,  40000, 40000, 0, 0, "t4");
  new_simple_task (ts,  9000,  50000, 50000, 0, 0, "t5");
  new_simple_task (ts,  5000,  50000, 50000, 0, 0, "t6");
  new_simple_task (ts,  8000,  59000, 59000, 0, 0, "t7");
  new_simple_task (ts, 12000,  80000, 80000, 0, 0, "t8");
  new_simple_task (ts,  2000,  80000, 80000, 0, 0, "t9");

  make_all_preemptible (ts);
  set_priorities (ts, DM);
  feas = feasible (ts, TRUE);

  print_task_set (ts);

  if (feas == num_tasks (ts)) {
    printf ("task set is feasible\n");
  } else {
    printf ("task set is not feasible\n");
  }

  simulate (ts, 100000000, "sim_output.txt", 0.0, NULL, NULL);

  for (i=0; i<=9; i++) {
    time_value orig = get_wcet (ts, i);
    double scale;

    for (scale = 1.0; scale > 0; scale -= 0.001) {
      set_wcet (ts, i, (int)(orig * scale));
      if (feasible (ts, FALSE) == num_tasks (ts)) {
	printf ("task %d : WCET must be scaled by %f to make the set feasible\n",
		i, scale);
	goto out;
      }
    }
    printf ("task %d: no scaling of this task can make the set feasible!\n", i);
  out:
    set_wcet (ts, i, orig);
  }

  return 0;
}
