/*
 * Copyright (c) 2002 University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of SPAK.
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation is hereby granted without fee, provided that the
 * above copyright notice and this permission/disclaimer notice is
 * retained in all copies or modified versions, and that both notices
 * appear in supporting documentation.  THE COPYRIGHT HOLDERS PROVIDE
 * THIS SOFTWARE "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE COPYRIGHT
 * HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER
 * RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu
 * any improvements that they make and grant redistribution rights to the
 * University of Utah.
 *
 * Author: John Regehr (regehr@cs.utah.edu)
 */

#include "spak_public.h"

/*
 * this file contains a bunch of random things I used spak for at one
 * time or another...
 */

#define DBG_LEVEL 1

int do_work (void);
static struct task_set *select_task_set (int which);

int main (void)
{
  struct task_set *ts;

  spak_srand ();
  
  switch (11) {

  case 1:
    ANNEAL_MAX = 1000;
    INIT_TEMP = 0.03;
    TEMP_SCALE = 0.98;

    // lots of random task sets
    random_preemp_thresh_tests ();
    break;

  case 2:
    // avionics task set from Locke, Tindell, etc.
    max_resp = 1000000;
    ANNEAL_MAX = 100000;
    INIT_TEMP = 0.04;
    TEMP_SCALE = 0.9995;
    
    DBGPrint (3, ("INIT_TEMP = %f, TEMP_SCALE = %f\n", INIT_TEMP, TEMP_SCALE));

    ts = select_task_set (40);
    preemption_thresh_test (ts, 0);
    break;

  case 3:
    // random task sets for Wang and Saksena vs. Regehr PT analysis bakeoff
    break_orig_preempt_thresh_analysis ();
    break;

  case 4:
    // compare PT analyses for one of the worst simple task sets found using case 3
    max_resp = 1000000;
    ts = select_task_set (41);
    compare_preempt_thresh_analyses (ts);
    break;

  case 5:
    max_resp = 10000000;
    ts = select_task_set (42);
    assert (feasible (ts, TRUE) == num_tasks (ts));
    simulate (ts, 100000, "sim_output.txt", 0.0, NULL, stdout);
    break;

  case 6:
    ANNEAL_MAX = 50000;
    INIT_TEMP = 0.10;
    TEMP_SCALE = 0.9999;
    max_resp = 1000 * 100; 
    find_hg_task_set ();
    break;

  case 7:
    avionics_test ();
    break;
      
  case 8:
    max_resp = 10000000;
    ANNEAL_MAX = 700;
    INIT_TEMP = 0.03;
    TEMP_SCALE = 0.97;
    ts = select_task_set (43);
    assert (!has_task_clusters (ts));
    assert (!has_task_barriers (ts));
    preemption_thresh_test (ts, 0);
    break;

  case 9:
    max_resp = 10000000;
    {
      int i;
      int f1, f2;
      int good = 0, bad = 0;
      for (i=0; i<100000; i++) {
	struct task_set *ts2;
	ts = create_random_task_set (i,
				     10, 
				     1000, 
				     "Wang00",
				     1, // multiplier
				     0, // has jitter
				     FALSE // independent period
				     );
	set_priorities (ts, INORDER);
	assign_random_preempt_thresh (ts);

	f1 = feasible (ts, TRUE);

	ts2 = copy_task_set (ts);
	set_analysis (ts2, "Wang00_fixed");

	f2 = feasible (ts2, TRUE);

	//print_task_set (ts);
	//print_task_set (ts2);

	if (ensure_same_response_times (ts, ts2)) {
	  good++;
	} else {
	  bad++;
	}

	free_task_set (ts);
	free_task_set (ts2);
      }
      printf ("%d good, %d bad\n", good, bad);
    }
    break;

  case 10:
    {
      double cs;
      struct task_set *ts2;

      max_resp = 1000000;
      ts = select_task_set (8);
      set_priorities (ts, DM);
      assert (feasible (ts, TRUE) == num_tasks (ts));
      printf ("original task set:\n");
      print_task_set (ts);
      printf ("\n");

      cs = find_critical_scale (ts, &ts2);
      printf ("critical scaling factor is %f\n", cs);
      printf ("scaled task set with DM schedule:\n");
      print_task_set (ts2);
      free_task_set (ts2);
      printf ("\n");

      set_pri (ts, 0, 1);
      set_pri (ts, 1, 0);
      make_all_preemptible (ts);

      cs = find_critical_scale (ts, &ts2);
      printf ("critical scaling factor is %f\n", cs);
      printf ("scaled task set with non-DM schedule:\n");
      print_task_set (ts2);
      free_task_set (ts2);
      printf ("\n");
    }      
    free_task_set (ts);
    break;

  case 11:
    {
      const int INC = 1000;
      int t1time, t2time;
      struct task_cluster *c;

      ANNEAL_MAX = 1000;
      INIT_TEMP = 0.03;
      TEMP_SCALE = 0.98;

      max_resp = 10000000;
      
      for (t1time = INC; t1time<=30000; t1time += INC) {
	for (t2time = INC; t2time<=30000; t2time += INC) {
	  ts = create_task_set (6, 0, 0, 5, "foo", 1000, 0, 0, 0, "Wang00_fixed");
	  new_simple_task (ts,  150, 7000, 7000, 0, 0, "t1");
	  new_simple_task (ts,   17, 7000, 7000, 0, 0, "t2");
	  new_simple_task (ts, 3160, 7000, 7000, 0, 0, "t3");
	  new_simple_task (ts, t1time, 250000, 250000, 0, 0, "bg1");
	  new_simple_task (ts, t2time,  50000,  50000, 0, 0, "bg2");

	  c = new_task_cluster (ts, "clus");
	  add_to_task_cluster (ts, c, "t1");
	  add_to_task_cluster (ts, c, "t2");
	  add_to_task_cluster (ts, c, "t3");
	  create_clusters_for_singletons (ts);

	  set_priorities (ts, INORDER);
	  make_all_preemptible (ts);

	  printf ("%d %d ", t1time, t2time);

	  if (anneal_priorities_and_thresholds (&ts, FALSE)) {
	    printf ("yes ");
	    minimize_threads_by_annealing (ts, FALSE, FALSE);
	  } else {
	    printf ("no");
	    free_task_set (ts);
	  }

	  printf ("\n");

	}
      }
    }
    break;
    
  default:
    assert (0);
  }
  
  print_xmalloc_cnt ();

  DBGPrint (0, (" *** normal termination ***\n"));

  return 0;
}

static struct task_set *select_task_set (int which)
{
  struct task_set *ts = NULL;

  switch (which) {

  case 1:
    /* Liu and Layland worst-case for two tasks */
    ts = create_task_set (2, 0, 0, 0,
			  "liu_layland_2tasks", 
			  1000, 0, 0, 0, 
			  "Audsley92");
    new_task (ts, 414, 1000, 1000, 1, 1000, 0, 0, "t1");
    new_task (ts, 586, 1414, 1414, 1, 1414, 0, 0, "t2");
    make_all_preemptible (ts);
    break;

  case 2:
    /* task set from Table 1 of Audsley et al. 93 */
    ts = create_task_set (3, 0, 0, 0,
			  "audsley93_t1", 
			  1000, 0, 0, 0,
			  "Audsley92");
    new_task (ts, 1, 50, 50, 1, 4,  0, 0, "t1");
    new_task (ts, 2, 9,  9,  1, 6,  0, 0, "t2");
    new_task (ts, 5, 20, 20, 1, 12, 0, 0, "t3");
    make_all_preemptible (ts);
    break;

  case 3:
    /* task set from Table 1 of Audsley et al. 93, modified as
       suggested in the text */
    ts = create_task_set (3, 0, 0, 0,
			  "audsley93_t1_alt", 
			  1000, 0, 0, 0,
			  "Audsley92");
    new_task (ts, 3, 50, 50, 1, 4,  0, 0, "t1");
    new_task (ts, 2, 9,  9,  1, 6,  0, 0, "t2");
    new_task (ts, 5, 20, 20, 1, 12, 0, 0, "t3");
    make_all_preemptible (ts);
    break;
    
  case 4:
    /* task set from Table 3 of Audsley et al. 93 */
    ts = create_task_set (18, 10, 500, 0,
			  "audsley93_t3", 
			  1000, 0, 0, 0,
			  "Tindell92_restricted");
    new_task (ts,   51,    1000,    1000, 1,    1000,    0,    0, "t1");
    new_task (ts, 3000,  200000,  200000, 1,    5000,    0,  300, "t2");
    new_task (ts, 2000,   25000,   25000, 1,   25000,    0,  600, "t3");
    new_task (ts, 5000,   25000,   25000, 1,   25000,    0,  900, "t4");
    new_task (ts, 1000,   40000,   40000, 1,   40000,    0, 1350, "t5");
    new_task (ts, 3000,   50000,   50000, 1,   50000,    0, 1350, "t6");
    new_task (ts, 5000,   50000,   50000, 1,   50000,    0,  750, "t7");
    new_task (ts, 8000,   59000,   59000, 1,   59000,    0,  750, "t8");
    new_task (ts, 9000,   80000,   80000, 1,   80000,    0, 1350, "t9");
    new_task (ts, 2000,   80000,   80000, 1,   80000,    0,  450, "t10");
    new_task (ts, 5000,  100000,  100000, 1,  100000,    0, 1050, "t11");
    new_task (ts, 1000,  200000,  200000, 1,  200000, 1000,  450, "t12");
    new_task (ts, 3000,  200000,  200000, 1,  200000,    0,  450, "t13");
    new_task (ts, 1000,  200000,  200000, 1,  200000,    0,  450, "t14");
    new_task (ts, 1000,  200000,  200000, 1,  200000,    0, 1350, "t15");
    new_task (ts, 3000,  200000,  200000, 1,  200000,    0,    0, "t16");
    new_task (ts, 1000, 1000000, 1000000, 1, 1000000,    0,    0, "t17");
    new_task (ts, 1000, 1000000, 1000000, 1, 1000000,    0,    0, "t18");
    make_all_preemptible (ts);
    break;

  case 5:
    /* task set E1 from Dual Priority Assignment */
    ts = create_task_set (3, 0, 0, 0,
			  "dual_priority_e1", 
			  1000, 0, 0, 0,
			  "Tindell92_restricted");
    new_task (ts, 3000,  6000,  6000, 1,  6000, 0, 0, "t1");
    new_task (ts, 2000,  8000,  8000, 1,  8000, 0, 0, "t2");
    new_task (ts, 3000, 12000, 12000, 1, 12000, 0, 0, "t3");
    make_all_preemptible (ts);
    break;

  case 6:
    /* task set E2 from Dual Priority Assignment */
    ts = create_task_set (4, 0, 0, 0,
			  "dual_priority_e2", 
			  1000, 0, 0, 0,
			  "Tindell92_restricted");
    new_task (ts, 3, 12, 12, 1, 12, 0, 0, "t1");
    new_task (ts, 4, 16, 16, 1, 16, 0, 0, "t2");
    new_task (ts, 4, 20, 20, 1, 20, 0, 0, "t3");
    new_task (ts, 6, 20, 20, 1, 20, 0, 0, "t4");
    make_all_preemptible (ts);
    break;

  case 7:
    /* task set from Tindell 92 */
    ts = create_task_set (18, 5, 10, 0,
			  "tindell92_gap", 
			  1000, 66, 74, 40,
			  "Tindell92_general");
    //               C        T        t  n        D     J   B
    new_task (ts, 3000,  200000,  200000, 1,    5000,    0,  0, "t1");
    new_task (ts,  700,   25000,    5000, 3,    5000,    0,  0, "t2");
    new_task (ts, 1400,   25000,    5000, 3,   12000,    0,  0, "t3");
    new_task (ts, 1000,   40000,   40000, 1,   40000,    0,  0, "t4");
    new_task (ts, 3000,   50000,   50000, 1,   50000,    0,  0, "t5");
    new_task (ts, 5000,   50000,   50000, 1,   50000,    0,  0, "t6");
    new_task (ts, 8000,   59000,   59000, 1,   59000,    0,  0, "t7");
    new_task (ts, 9000,   80000,   80000, 1,   80000,    0,  0, "t8");
    new_task (ts, 2000,   80000,   80000, 1,  100000,    0,  0, "t9");
    new_task (ts, 5000,  100000,  100000, 1,  115000,    0,  0, "t10");
    new_task (ts, 1000,  200000,  200000, 1,  200000, 1000,  0, "t11");
    new_task (ts, 3000,  200000,  200000, 1,  200000,    0,  0, "t12");
    new_task (ts, 1000,  200000,  200000, 1,  200000,    0,  0, "t13");
    new_task (ts, 1000,  200000,  200000, 1,  200000,    0,  0, "t14");
    new_task (ts, 3000,  200000,  200000, 1,  200000,    0,  0, "t15");
    new_task (ts, 1000, 1000000, 1000000, 1, 1000000,    0,  0, "t16");
    new_task (ts, 1000, 1000000, 1000000, 1, 1000000,    0,  0, "t17");
    make_all_preemptible (ts);
    new_sem (ts, "sem1");
    new_sem (ts, "sem2");
    new_sem (ts, "sem3");
    new_sem (ts, "sem4");
    new_sem (ts, "sem5");
    new_lock (ts, "sem1",  "t9",  900);
    new_lock (ts, "sem2",  "t9",  300);
    new_lock (ts, "sem2", "t15", 1350);
    new_lock (ts, "sem3", "t10",  400);
    new_lock (ts, "sem3",  "t6",  400);
    new_lock (ts, "sem4",  "t9",  300);
    new_lock (ts, "sem4",  "t3",  100);
    new_lock (ts, "sem5", "t11",  750);
    new_lock (ts, "sem5", "t15",  750);
    break;

  case 8:
    /* motivation for maximizing insensitivity */
    ts = create_task_set (2, 0, 0, 0,
			  "very_sensitive", 
			  1000, 0, 0, 0,
			  "Tindell92_restricted");
    //               C        T        t  n        D     J   B
    new_task (ts,  400,    1999,    1999, 1,    1999,    0,  0, "t1");
    new_task (ts,  400,    2000,    2000, 1,    2000, 1200,  0, "t2");
    make_all_preemptible (ts);
    break;

  case 9:
    /* Table 1 from Wang and Saksena RTCSA preemption threshold paper */
    ts = create_task_set (3, 0, 0, 3,
			  "preemption_thresh_rtcsa_table1", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    //                    C    T    D    J    B
    new_simple_task (ts, 20,  70,  50,   0,   0, "t1");
    new_simple_task (ts, 20,  80,  80,   0,   0, "t2");
    new_simple_task (ts, 35, 200, 100,   0,   0, "t3");
    set_priorities (ts, DM);
    make_all_preemptible (ts);
    break;

  case 10:
    /* This task set breaks original preemption threhsold analysis */
    ts = create_task_set (3, 0, 0, 0,
			  "preemption_thresh_bad", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    //                    C    T    D    J    B
    new_simple_task (ts, 10, 210, 210,   0,   0, "t1");
    new_simple_task (ts, 60, 140, 140,   0,   0, "t2");
    new_simple_task (ts, 50, 115, 115,   0,   0, "t3");
    set_priorities (ts, DM);
    make_all_preemptible (ts);
    break;

  case 11:
    /* 
     * This task set breaks "fixed" preemption threshold analysis as of 2/20/2002 
     * (correction -- it breaks the simulation, which is now fixed)
     */
    ts = create_task_set (3, 0, 0, 0,
			  "preemption_thresh_bad", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    //                    C    T    D    J    B
    new_simple_task (ts, 40, 125, 170,   0,   0, "t1");
    new_simple_task (ts, 10,  35,  15,   0,   0, "t2");
    new_simple_task (ts, 70, 185, 175,   0,   0, "t3");
    set_priorities (ts, DM);
    make_all_preemptible (ts);
    break;

  case 12:
    /* 
     * This task set breaks "fixed" preemption threshold analysis as of 2/20/2002 
     * (correction -- it breaks the simulation, which is now fixed)
     */
    ts = create_task_set (3, 0, 0, 0,
			  "preemption_thresh_bad", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    //                    C    T    D    J    B
    new_simple_task (ts, 80, 215, 130,   0,   0, "t1");
    new_simple_task (ts,  5,  40,  40,   0,   0, "t2");
    new_simple_task (ts, 20,  70, 135,   0,   0, "t3");
    set_priorities (ts, DM);
    make_all_preemptible (ts);
    break;

  case 13:
    /* 
     * This task set breaks "fixed" preemption threshold analysis as of 2/20/2002 
     * (correction -- it breaks the simulation, which is now fixed)
     */
    ts = create_task_set (4, 0, 0, 0,
			  "preemption_thresh_bad", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    //                    C    T    D    J    B
    new_simple_task (ts, 45, 170, 215,   0,   0, "t1");
    new_simple_task (ts, 15,  60, 125,   0,   0, "t2");
    new_simple_task (ts, 75, 235, 225,   0,   0, "t3");
    new_simple_task (ts, 25, 155, 220,   0,   0, "t4");
    set_priorities (ts, DM);
    make_all_preemptible (ts);
    break;

  case 14:
    /* 
     * As of Feb 21 this task set breaks the optimal priority and threshold
     * assignment algorithm.
     */
    ts = create_task_set (3, 0, 0, 0,
			  "preemption_thresh_bad", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    //                    C    T    D    J    B
    new_simple_task (ts, 65, 185, 130,   0,   0, "t1");
    new_simple_task (ts, 30, 240,  55,   0,   0, "t2");
    new_simple_task (ts, 10,  75,  25,   0,   0, "t3");
    set_priorities (ts, DM);
    make_all_preemptible (ts);
    break;

  case 15:
    /* 
     * As of Feb 21 this task set breaks the optimal priority and threshold
     * assignment algorithm.
     */
    ts = create_task_set (5, 0, 0, 0,
			  "preemption_thresh_bad", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    //                    C    T    D    J    B
    new_simple_task (ts,  0, 105, 120,   0,   0, "t1");
    new_simple_task (ts,  1, 125, 110,   0,   0, "t2");
    new_simple_task (ts,  2, 110, 160,   0,   0, "t3");
    new_simple_task (ts,  0,  30, 225,   0,   0, "t4");
    new_simple_task (ts,  1,  55,   5,   0,   0, "t5");
    make_all_preemptible (ts);
    set_priorities (ts, INORDER);
    set_preempt_thresh (ts, 0, 2);
    set_preempt_thresh (ts, 1, 3);
    set_preempt_thresh (ts, 2, 1);
    set_preempt_thresh (ts, 3, 0);
    set_preempt_thresh (ts, 4, 0);
    break;

  case 16:
    ts = create_task_set (5, 0, 0, 0,
			  "preemption_thresh_bad", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    //                    C    T    D    J    B
    new_simple_task (ts, 50, 190,  85,   0,   0, "t1");
    new_simple_task (ts, 25, 100, 210,   0,   0, "t2");
    new_simple_task (ts, 25, 165,  65,   0,   0, "t3");
    new_simple_task (ts, 20, 135, 155,   0,   0, "t4");
    new_simple_task (ts, 10,  55, 230,   0,   0, "t5");
    set_priorities (ts, DM);
    make_all_preemptible (ts);
    break;

  case 17:
    /* This task set breaks original preemption threhsold analysis */
    ts = create_task_set (3, 0, 0, 0,
			  "preemption_thresh_bad", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    /* 
     * task set that is very much does not maximize insensitivity
     * with Audsley or DM
     */
    new_task (ts, 2,  6,  6, 1,  6, 0, 0, "t0");
    new_task (ts, 2,  8,  8, 1,  8, 0, 0, "t1");
    new_task (ts, 1,  9,  9, 1,  9, 0, 0, "t2");
    set_priorities (ts, DM);
    make_all_preemptible (ts);
    break;

  case 18:
    /* 
     * This task set breaks preemption thresh maximization as of
     * 2/24/02. 
     *
     * Note: the zero WCET screw up the routine for maximizing
     * preemption thresholds. 
     */
    ts = create_task_set (10, 0, 0, 0,
			  "preemption_thresh_bad", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    /* 
     * task set that is very much does not maximize insensitivity
     * with Audsley or DM
     */
    //                     C     T    D  J  B
    new_simple_task (ts, 100,  810, 215, 0, 0, "t0");
    new_simple_task (ts,  10, 1000,  45, 0, 0, "t1");
    new_simple_task (ts,   0,   15, 275, 0, 0, "t2");
    new_simple_task (ts, 170,  955, 330, 0, 0, "t3");
    new_simple_task (ts,   0,  235, 565, 0, 0, "t4");
    new_simple_task (ts,  75,  930, 490, 0, 0, "t5");
    new_simple_task (ts,  10,  415, 770, 0, 0, "t6");
    new_simple_task (ts,  35,  435, 180, 0, 0, "t7");
    new_simple_task (ts,  40,  570, 655, 0, 0, "t8");
    new_simple_task (ts,  10,  120, 335, 0, 0, "t9");
    set_priorities (ts, DM);
    make_all_preemptible (ts);
    break;

  case 19:
    /* 
     * This task set is a soso example for sensitivity analysis.
     */
    ts = create_task_set (10, 0, 0, 0,
			  "ts19_sensitivity_good", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 80, 1220, 8145, 0, 0, "t0");
    new_simple_task (ts, 135, 3850, 7805, 0, 0, "t1");
    new_simple_task (ts, 805, 7445, 2120, 0, 0, "t2");
    new_simple_task (ts, 170, 8745, 3010, 0, 0, "t3");
    new_simple_task (ts, 75, 7175, 5840, 0, 0, "t4");
    new_simple_task (ts, 730, 8880, 6655, 0, 0, "t5");
    new_simple_task (ts, 55, 780, 2195, 0, 0, "t6");
    new_simple_task (ts, 1135, 8610, 7965, 0, 0, "t7");
    new_simple_task (ts, 20, 645, 7215, 0, 0, "t8");
    new_simple_task (ts, 175, 2440, 4535, 0, 0, "t9");
    set_priorities (ts, DM);
    make_all_preemptible (ts);
    break;

  case 20:
    /* 
     * This task set is a good example for sensitivity analysis.
     */
    ts = create_task_set (10, 0, 0, 0,
			  "ts20_sensitivity_good", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 475, 3905, 9700, 0, 0, "t0");
    new_simple_task (ts, 50, 1615, 3610, 0, 0, "t1");
    new_simple_task (ts, 275, 4015, 990, 0, 0, "t2");
    new_simple_task (ts, 80, 1190, 6975, 0, 0, "t3");
    new_simple_task (ts, 130, 8560, 330, 0, 0, "t4");
    new_simple_task (ts, 360, 6035, 545, 0, 0, "t5");
    new_simple_task (ts, 1005, 9545, 3935, 0, 0, "t6");
    new_simple_task (ts, 25, 705, 290, 0, 0, "t7");
    new_simple_task (ts, 950, 4945, 9805, 0, 0, "t8");
    new_simple_task (ts, 110, 9155, 3115, 0, 0, "t9");
    break;

  case 21:
    /* 
     * This task set is a good example for sensitivity analysis.
     */
    ts = create_task_set (10, 0, 0, 0,
			  "ts21_sensitivity_good", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 105, 1845, 1780, 0, 0, "t0");
    new_simple_task (ts, 240, 7495, 5420, 0, 0, "t1");
    new_simple_task (ts, 260, 7545, 9875, 0, 0, "t2");
    new_simple_task (ts, 640, 3225, 6505, 0, 0, "t3");
    new_simple_task (ts, 80, 3780, 9925, 0, 0, "t4");
    new_simple_task (ts, 600, 7890, 9610, 0, 0, "t5");
    new_simple_task (ts, 265, 3245, 4830, 0, 0, "t6");
    new_simple_task (ts, 305, 3900, 5220, 0, 0, "t7");
    new_simple_task (ts, 1660, 9280, 5685, 0, 0, "t8");
    new_simple_task (ts, 95, 7690, 7590, 0, 0, "t9");
    break;

  case 22:
    /* 
     * This task set is a good example for sensitivity analysis.
     */
    ts = create_task_set (10, 0, 0, 0,
			  "ts22_sensitivity_good", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 445, 9985, 5980, 0, 0, "t0");
    new_simple_task (ts, 360, 7260, 6330, 0, 0, "t1");
    new_simple_task (ts, 215, 7285, 275, 0, 0, "t2");
    new_simple_task (ts, 310, 5140, 1115, 0, 0, "t3");
    new_simple_task (ts, 80, 4380, 6625, 0, 0, "t4");
    new_simple_task (ts, 360, 3085, 5955, 0, 0, "t5");
    new_simple_task (ts, 315, 8755, 860, 0, 0, "t6");
    new_simple_task (ts, 110, 2120, 2085, 0, 0, "t7");
    new_simple_task (ts, 370, 7500, 6150, 0, 0, "t8");
    new_simple_task (ts, 250, 3435, 8060, 0, 0, "t9");
    break;

  case 23:
    /* 
     * This task set is a good example for sensitivity analysis.
     */
    ts = create_task_set (10, 0, 0, 0,
			  "ts23_sensitivity_good", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 455, 6540, 1270, 0, 0, "t0");
    new_simple_task (ts, 370, 6080, 6220, 0, 0, "t1");
    new_simple_task (ts, 60, 2570, 2480, 0, 0, "t2");
    new_simple_task (ts, 160, 1825, 630, 0, 0, "t3");
    new_simple_task (ts, 5, 145, 2695, 0, 0, "t4");
    new_simple_task (ts, 1060, 9940, 4125, 0, 0, "t5");
    new_simple_task (ts, 1025, 8320, 6415, 0, 0, "t6");
    new_simple_task (ts, 405, 2925, 4760, 0, 0, "t7");
    new_simple_task (ts, 595, 9415, 6580, 0, 0, "t8");
    new_simple_task (ts, 20, 1115, 2130, 0, 0, "t9");
    break;

  case 24:
    /* 
     * This task set is a good example for sensitivity analysis.
     */
    ts = create_task_set (10, 0, 0, 0,
			  "ts24_sensitivity_good", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 1515, 9145, 2895, 0, 0, "t0");
    new_simple_task (ts, 280, 6690, 9710, 0, 0, "t1");
    new_simple_task (ts, 115, 5005, 9450, 0, 0, "t2");
    new_simple_task (ts, 20, 640, 5320, 0, 0, "t3");
    new_simple_task (ts, 60, 470, 1555, 0, 0, "t4");
    new_simple_task (ts, 640, 8760, 8405, 0, 0, "t5");
    new_simple_task (ts, 125, 1390, 3315, 0, 0, "t6");
    new_simple_task (ts, 10, 235, 4890, 0, 0, "t7");
    new_simple_task (ts, 175, 1650, 3925, 0, 0, "t8");
    new_simple_task (ts, 140, 5660, 8915, 0, 0, "t9");
    break;

  case 25:
    /* 
     * This task set is a good example for sensitivity analysis.
     */
    ts = create_task_set (10, 0, 0, 0,
			  "ts25_sensitivity_good", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 600, 5195, 4110, 0, 0, "t0");
    new_simple_task (ts, 40, 2395, 7220, 0, 0, "t1");
    new_simple_task (ts, 970, 6655, 3080, 0, 0, "t2");
    new_simple_task (ts, 165, 2550, 6275, 0, 0, "t3");
    new_simple_task (ts, 575, 3710, 2425, 0, 0, "t4");
    new_simple_task (ts, 120, 3625, 6995, 0, 0, "t5");
    new_simple_task (ts, 175, 3610, 5460, 0, 0, "t6");
    new_simple_task (ts, 165, 9800, 8190, 0, 0, "t7");
    new_simple_task (ts, 1080, 6405, 3785, 0, 0, "t8");
    new_simple_task (ts, 315, 5625, 9310, 0, 0, "t9");
    break;

  case 26:
    /* 
     * This task set is a good example for sensitivity analysis.
     */
    ts = create_task_set (10, 0, 0, 0,
			  "ts26_sensitivity_good", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 450, 3310, 9000, 0, 0, "t0");
    new_simple_task (ts, 265, 7890, 6225, 0, 0, "t1");
    new_simple_task (ts, 175, 7855, 5905, 0, 0, "t2");
    new_simple_task (ts, 540, 4215, 6330, 0, 0, "t3");
    new_simple_task (ts, 1260, 9515, 8445, 0, 0, "t4");
    new_simple_task (ts, 980, 5690, 6550, 0, 0, "t5");
    new_simple_task (ts, 170, 5230, 4345, 0, 0, "t6");
    new_simple_task (ts, 10, 1335, 1700, 0, 0, "t7");
    new_simple_task (ts, 795, 7340, 9340, 0, 0, "t8");
    new_simple_task (ts, 135, 970, 9685, 0, 0, "t9");
    break;

  case 27:
    /* 
     * This task set is a good example for sensitivity analysis.
     */
    ts = create_task_set (10, 0, 0, 0,
			  "ts27_sensitivity_good", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 305, 4135, 6065, 0, 0, "t0");
    new_simple_task (ts, 45, 2675, 7365, 0, 0, "t1");
    new_simple_task (ts, 495, 8405, 7855, 0, 0, "t2");
    new_simple_task (ts, 225, 8410, 6865, 0, 0, "t3");
    new_simple_task (ts, 460, 9155, 2570, 0, 0, "t4");
    new_simple_task (ts, 5, 250, 3550, 0, 0, "t5");
    new_simple_task (ts, 155, 2205, 1595, 0, 0, "t6");
    new_simple_task (ts, 120, 4865, 3740, 0, 0, "t7");
    new_simple_task (ts, 1255, 7535, 1605, 0, 0, "t8");
    new_simple_task (ts, 150, 4570, 1870, 0, 0, "t9");
    break;

  case 28:
    /* 
     * This task set is a good example for sensitivity analysis.
     */
    ts = create_task_set (10, 0, 0, 10,
			  "ts28_sensitivity_good", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 75, 1015, 7165, 0, 0, "t0");
    new_simple_task (ts, 230, 3875, 3050, 0, 0, "t1");
    new_simple_task (ts, 430, 2445, 2215, 0, 0, "t2");
    new_simple_task (ts, 50, 1825, 2815, 0, 0, "t3");
    new_simple_task (ts, 255, 2200, 9950, 0, 0, "t4");
    new_simple_task (ts, 1340, 8340, 4045, 0, 0, "t5");
    new_simple_task (ts, 175, 8010, 1580, 0, 0, "t6");
    new_simple_task (ts, 250, 1475, 7495, 0, 0, "t7");
    new_simple_task (ts, 80, 1750, 4280, 0, 0, "t8");
    new_simple_task (ts, 855, 7320, 1285, 0, 0, "t9");
    break;

  case 29:
    /* 
     * This task set is a good example for sensitivity analysis.
     */
    ts = create_task_set (10, 0, 0, 10,
			  "ts29_sensitivity_good", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 35, 2515, 2710, 0, 0, "t0");
    new_simple_task (ts, 90, 2570, 9585, 0, 0, "t1");
    new_simple_task (ts, 1255, 7340, 1655, 0, 0, "t2");
    new_simple_task (ts, 220, 3225, 8955, 0, 0, "t3");
    new_simple_task (ts, 725, 6645, 3855, 0, 0, "t4");
    new_simple_task (ts, 170, 1210, 5940, 0, 0, "t5");
    new_simple_task (ts, 130, 2710, 2620, 0, 0, "t6");
    new_simple_task (ts, 60, 2440, 2025, 0, 0, "t7");
    new_simple_task (ts, 235, 5755, 1930, 0, 0, "t8");
    new_simple_task (ts, 110, 4090, 4855, 0, 0, "t9");
    break;

  case 30:
    /* 
     * This task set is a good example for sensitivity analysis.
     */
    ts = create_task_set (10, 0, 0, 10,
			  "ts30_sensitivity_good", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 170, 2960, 9180, 0, 0, "t0");
    new_simple_task (ts, 700, 9215, 4745, 0, 0, "t1");
    new_simple_task (ts, 85, 2085, 5750, 0, 0, "t2");
    new_simple_task (ts, 135, 1795, 255, 0, 0, "t3");
    new_simple_task (ts, 745, 5125, 4780, 0, 0, "t4");
    new_simple_task (ts, 125, 1435, 1725, 0, 0, "t5");
    new_simple_task (ts, 65, 3480, 1865, 0, 0, "t6");
    new_simple_task (ts, 490, 2775, 7450, 0, 0, "t7");
    new_simple_task (ts, 260, 9045, 3465, 0, 0, "t8");
    new_simple_task (ts, 1400, 7770, 2030, 0, 0, "t9");
    break;

  case 31:
    /* 
     * This task set is a good example for sensitivity analysis.
     */
    ts = create_task_set (10, 0, 0, 10,
			  "ts31_sensitivity_good", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 5, 75, 2655, 0, 0, "t0");
    new_simple_task (ts, 75, 1545, 1825, 0, 0, "t1");
    new_simple_task (ts, 150, 8065, 1400, 0, 0, "t2");
    new_simple_task (ts, 190, 1940, 760, 0, 0, "t3");
    new_simple_task (ts, 745, 8265, 9710, 0, 0, "t4");
    new_simple_task (ts, 115, 1460, 6165, 0, 0, "t5");
    new_simple_task (ts, 845, 6755, 2135, 0, 0, "t6");
    new_simple_task (ts, 530, 6500, 9420, 0, 0, "t7");
    new_simple_task (ts, 305, 9555, 7365, 0, 0, "t8");
    new_simple_task (ts, 220, 6180, 8930, 0, 0, "t9");
    break;

  case 32:
    /* 
     * This task set is a good example for sensitivity analysis.
     */
    ts = create_task_set (10, 0, 0, 10,
			  "ts32_sensitivity_good", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 210, 4155, 4095, 0, 0, "t0");
    new_simple_task (ts, 110, 3045, 7120, 0, 0, "t1");
    new_simple_task (ts, 710, 6385, 1780, 0, 0, "t2");
    new_simple_task (ts, 380, 4745, 5995, 0, 0, "t3");
    new_simple_task (ts, 110, 4405, 5385, 0, 0, "t4");
    new_simple_task (ts, 410, 3670, 7870, 0, 0, "t5");
    new_simple_task (ts, 900, 7415, 8070, 0, 0, "t6");
    new_simple_task (ts, 510, 8970, 5315, 0, 0, "t7");
    new_simple_task (ts, 75, 2105, 530, 0, 0, "t8");
    new_simple_task (ts, 400, 3480, 6195, 0, 0, "t9");
    break;

  case 33:
    /* 
     * This task set is a good example for sensitivity analysis.
     */
    ts = create_task_set (10, 0, 0, 10,
			  "ts33_sensitivity_good", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 335, 4800, 5485, 0, 0, "t0");
    new_simple_task (ts, 335, 9890, 7770, 0, 0, "t1");
    new_simple_task (ts, 55, 1100, 9400, 0, 0, "t2");
    new_simple_task (ts, 5, 55, 805, 0, 0, "t3");
    new_simple_task (ts, 1430, 8540, 2140, 0, 0, "t4");
    new_simple_task (ts, 635, 3760, 8830, 0, 0, "t5");
    new_simple_task (ts, 870, 8040, 7525, 0, 0, "t6");
    new_simple_task (ts, 265, 7005, 5205, 0, 0, "t7");
    new_simple_task (ts, 355, 9960, 1960, 0, 0, "t8");
    new_simple_task (ts, 150, 7785, 2570, 0, 0, "t9");
    break;

  case 34:
    /* 
     * This task set is a soso example for sensitivity analysis.
     */
    ts = create_task_set (10, 0, 0, 10,
			  "ts34_sensitivity_good", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 770, 6780, 3535, 0, 0, "t0");
    new_simple_task (ts, 15, 540, 3945, 0, 0, "t1");
    new_simple_task (ts, 110, 1400, 9510, 0, 0, "t2");
    new_simple_task (ts, 55, 3705, 1405, 0, 0, "t3");
    new_simple_task (ts, 1485, 8965, 2890, 0, 0, "t4");
    new_simple_task (ts, 590, 9435, 3870, 0, 0, "t5");
    new_simple_task (ts, 515, 4355, 1115, 0, 0, "t6");
    new_simple_task (ts, 25, 535, 9755, 0, 0, "t7");
    new_simple_task (ts, 620, 6565, 6360, 0, 0, "t8");
    new_simple_task (ts, 145, 7225, 6360, 0, 0, "t9");
    break;

  case 35:
    /* 
     * This task set is a good example for sensitivity analysis --
     * consider using this one in the paper.  
     */
    ts = create_task_set (10, 0, 0, 10,
			  "ts35_sensitivity_good", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 105, 1845, 1780, 0, 0, "t0");
    new_simple_task (ts, 240, 7495, 5420, 0, 0, "t1");
    new_simple_task (ts, 260, 7545, 9875, 0, 0, "t2");
    new_simple_task (ts, 640, 3225, 6505, 0, 0, "t3");
    new_simple_task (ts, 80, 3780, 9925, 0, 0, "t4");
    new_simple_task (ts, 600, 7890, 9610, 0, 0, "t5");
    new_simple_task (ts, 265, 3245, 4830, 0, 0, "t6");
    new_simple_task (ts, 305, 3900, 5220, 0, 0, "t7");
    new_simple_task (ts, 1660, 9280, 5685, 0, 0, "t8");
    new_simple_task (ts, 95, 7690, 7590, 0, 0, "t9");
    break;

  case 36:
    /* 
     * This task set is a soso example for sensitivity analysis.
     */
    ts = create_task_set (10, 0, 0, 10,
			  "ts36_sensitivity_good", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 805, 5110, 6930, 0, 0, "t0");
    new_simple_task (ts, 945, 8385, 7565, 0, 0, "t1");
    new_simple_task (ts, 70, 580, 2745, 0, 0, "t2");
    new_simple_task (ts, 30, 1290, 8485, 0, 0, "t3");
    new_simple_task (ts, 250, 5790, 9960, 0, 0, "t4");
    new_simple_task (ts, 150, 2325, 9820, 0, 0, "t5");
    new_simple_task (ts, 890, 9230, 7910, 0, 0, "t6");
    new_simple_task (ts, 105, 4150, 5745, 0, 0, "t7");
    new_simple_task (ts, 670, 8555, 1685, 0, 0, "t8");
    new_simple_task (ts, 335, 2080, 6050, 0, 0, "t9");
    break;
    
  case 37:
    /* task set from Table 3 of Audsley et al. 93 */
    ts = create_task_set (18, 10, 500, 18,
			  "audsley93_t3", 
			  1000, 0, 0, 0,
			  "Tindell92_restricted");
    new_task (ts,   51,    1000,    1000, 1,    1000,    0,    0, "t0");
    new_task (ts, 3000,  200000,  200000, 1,    5000,    0,  300, "t1");
    new_task (ts, 2000,   25000,   25000, 1,   25000,    0,  600, "t2");
    new_task (ts, 5000,   25000,   25000, 1,   25000,    0,  900, "t3");
    new_task (ts, 1000,   40000,   40000, 1,   40000,    0, 1350, "t4");
    new_task (ts, 3000,   50000,   50000, 1,   50000,    0, 1350, "t5");
    new_task (ts, 5000,   50000,   50000, 1,   50000,    0,  750, "t6");
    new_task (ts, 8000,   59000,   59000, 1,   59000,    0,  750, "t7");
    new_task (ts, 9000,   80000,   80000, 1,   80000,    0, 1350, "t8");
    new_task (ts, 2000,   80000,   80000, 1,   80000,    0,  450, "t9");
    new_task (ts, 5000,  100000,  100000, 1,  100000,    0, 1050, "t10");
    new_task (ts, 1000,  200000,  200000, 1,  200000, 1000,  450, "t11");
    new_task (ts, 3000,  200000,  200000, 1,  200000,    0,  450, "t12");
    new_task (ts, 1000,  200000,  200000, 1,  200000,    0,  450, "t13");
    new_task (ts, 1000,  200000,  200000, 1,  200000,    0, 1350, "t14");
    new_task (ts, 3000,  200000,  200000, 1,  200000,    0,    0, "t15");
    new_task (ts, 1000, 1000000, 1000000, 1, 1000000,    0,    0, "t16");
    new_task (ts, 1000, 1000000, 1000000, 1, 1000000,    0,    0, "t17");
#if 1
    {
      struct task_cluster *cluster;

      cluster = new_task_cluster (ts, "sem2");
      add_to_task_cluster (ts, cluster, "t9");
      add_to_task_cluster (ts, cluster, "t15");

      cluster = new_task_cluster (ts, "sem3");
      add_to_task_cluster (ts, cluster, "t6");
      add_to_task_cluster (ts, cluster, "t10");

      cluster = new_task_cluster (ts, "sem4");
      add_to_task_cluster (ts, cluster, "t9");
      add_to_task_cluster (ts, cluster, "t3");

      cluster = new_task_cluster (ts, "sem5");
      add_to_task_cluster (ts, cluster, "t11");
      add_to_task_cluster (ts, cluster, "t15");

    }
#endif
    break;

  case 38:
    ts = create_task_set (1, 0, 0, 1,
			  "audsley93_t3", 
			  1000, 0, 0, 0,
			  "Tindell92_restricted");
    //                   C   T   D   J  B
    new_simple_task (ts, 5, 10, 80, 33, 0, "t0");
    break;

  case 39:
    ts = create_task_set (1, 0, 0, 1,
			  "audsley93_t3", 
			  1000, 0, 0, 0,
			  "Tindell92_restricted");
    //                   C   T   D   J  B
    new_simple_task (ts, 10, 10, 55, 8, 0, "t0");
    break;

  case 40:
    /* task set from Tindell 92 */
    ts = create_task_set (18, 0, 0, 14,
			  "tindell92_gap", 
			  1000, 0, 0, 0,
			  "Tindell92_general");
    //                      C       T       D     J   B
    new_simple_task (ts,   51,   1000,   1000,    0,  0, "t0");
    new_simple_task (ts, 3000, 200000,   5000,    0,  0, "t1");
    new_simple_task (ts,  700,  25000,   5000,    0,  0, "t2");
    new_simple_task (ts, 1400,  25000,  12000,    0,  0, "t3");
    new_simple_task (ts, 1000,  40000,  40000,    0,  0, "t4");
    new_simple_task (ts, 3000,  50000,  50000,    0,  0, "t5");
    new_simple_task (ts, 5000,  50000,  50000,    0,  0, "t6");
    new_simple_task (ts, 8000,  59000,  59000,    0,  0, "t7");
    new_simple_task (ts, 9000,  80000,  80000,    0,  0, "t8");
    new_simple_task (ts, 2000,  80000, 100000,    0,  0, "t9");
    new_simple_task (ts, 5000, 100000, 115000,    0,  0, "t10");
    new_simple_task (ts, 1000, 200000, 200000, 1000,  0, "t11");
    new_simple_task (ts, 3000, 200000, 200000,    0,  0, "t12");
    new_simple_task (ts, 1000, 200000, 200000,    0,  0, "t13");
    new_simple_task (ts, 1000, 200000, 200000,    0,  0, "t14");
    new_simple_task (ts, 3000, 200000, 200000,    0,  0, "t15");
    new_simple_task (ts, 1000,1000000,1000000,    0,  0, "t16");
    new_simple_task (ts, 1000,1000000,1000000,    0,  0, "t17");
#if 1
    {
      struct task_cluster *cluster;

      cluster = new_task_cluster (ts, "sem2");
      add_to_task_cluster (ts, cluster, "t3");
      add_to_task_cluster (ts, cluster, "t9");
      add_to_task_cluster (ts, cluster, "t11");
      add_to_task_cluster (ts, cluster, "t15");

      cluster = new_task_cluster (ts, "sem3");
      add_to_task_cluster (ts, cluster, "t6");
      add_to_task_cluster (ts, cluster, "t10");

      new_task_barrier (ts, 0);
    }
#endif
    break;

  case 41:
    ts = create_task_set (3, 0, 0, 3,
			  "ts_breaks_pt", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts, 40,  70,  70, 0, 0, "t0");
    new_simple_task (ts, 20,  90,  90, 0, 0, "t1");
    new_simple_task (ts, 20, 100, 100, 0, 0, "t2");
    break;

  case 42:
    // this one breaks either the PT analysis or simulation as of 3/20/02
    ts = create_task_set (6, 0, 0, 6,
			  "ts_breaks_pt", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task_with_pri (ts, 10, 140, 410, 0, 0, 2, 2, "t0");
    new_simple_task_with_pri (ts, 40, 140, 510, 0, 0, 2, 2, "t1");
    new_simple_task_with_pri (ts, 120, 830, 320, 0, 0, 1, 0, "t2");
    new_simple_task_with_pri (ts, 30, 330, 800, 0, 0, 5, 4, "t3");
    new_simple_task_with_pri (ts, 100, 760, 530, 60, 0, 3, 3, "t4");
    new_simple_task_with_pri (ts, 10, 110, 830, 0, 0, 4, 4, "t5");
    break;

  case 43:
    ts = create_task_set (5, 0, 0, 5,
			  "ts_breaks_foo", 
			  1000, 0, 0, 0,
			  "Wang00_fixed");
    new_simple_task (ts,  26,    92,      546,     17, 0, "t0");
    new_simple_task (ts,  67,    216,     907,       0, 0, "t1");
    new_simple_task (ts,  15,    264,     599,      87, 0, "t2");
    new_simple_task (ts,  87,    709,     954,      88, 0, "t3");
    new_simple_task (ts,  84,    369,     159,       0, 0, "t4");
    break;
  
  default:
    assert (FALSE);
  }

  return ts;
}
