/*
 * Copyright (c) 2002 University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of SPAK.
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation is hereby granted without fee, provided that the
 * above copyright notice and this permission/disclaimer notice is
 * retained in all copies or modified versions, and that both notices
 * appear in supporting documentation.  THE COPYRIGHT HOLDERS PROVIDE
 * THIS SOFTWARE "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE COPYRIGHT
 * HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER
 * RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu
 * any improvements that they make and grant redistribution rights to the
 * University of Utah.
 *
 * Author: John Regehr (regehr@cs.utah.edu)
 */

#include "spak_public.h"

#define DBG_LEVEL 1

int compare_preempt_thresh_analyses (struct task_set *ts1)
{
  int res1, x1, x2;
  struct task_set *ts2;
  int w;
  static int worst = 0;

  printf ("U = %f\n", utilization_set (ts1));

  set_analysis (ts1, "Wang00");
  {
    struct task_set *xts = copy_task_set (ts1);
    res1 = anneal_priorities_and_thresholds (&xts, TRUE);
    if (res1) {
      ts1 = xts;
    }
  }
  // res1 = greedy_priorities_and_thresholds (ts1);
  if (!res1) {
    printf ("not schedulable using orig Wang analysis\n");
    return 0;
  }

  ts2 = copy_task_set (ts1);
  set_analysis (ts2, "Wang00_fixed");

  x1 = feasible (ts1, TRUE);
  x2 = feasible (ts2, TRUE);
  if (x2 == num_tasks (ts2)) {
    printf ("schedulable using fixed Wang analysis\n");
    free_task_set (ts2);
    return 0;
  }

  simulate (ts2, 10000000, NULL, 0.0, NULL, stdout);

  w = second_worse (ts1, ts2);
  if (w > 30) {  // silly hack
    worst = w;
    printf ("\n\n\n");
    print_task_set (ts1);
    print_task_set (ts2);
    printf ("worst is now %d\n", worst);

    save_task_set_source_code (ts1);
    save_task_set_latex (ts1);
    // this might bomb SPAK
    simulate (ts1, 10000000, "sim_output.txt", 0.0, NULL, stdout);
  }

  free_task_set (ts2);
  return 1;
}

void break_orig_preempt_thresh_analysis (void)
{
  struct task_set *ts;
  int rep, total_tasks, max_deadline, REPS, pct;
  
  max_deadline = 100;
  max_resp = max_deadline * 100; // seems sufficient...

  REPS = 100000;
  
  assert (max_resp < (INT_MAX / 2));

  pct = -1;
  rep = 0;
  do {
    if ((100*rep/REPS) > pct) {
      pct = 100*rep/REPS;
      DBGPrint (5, ("%d pct done (%d out of %d)\n", pct, rep, REPS));
    }
    total_tasks = 3;
    ts = create_random_task_set (rep,
				 total_tasks, 
				 max_deadline, 
				 "Wang00_fixed",
				 10, FALSE, FALSE);
    rep += compare_preempt_thresh_analyses (ts);
    free_task_set (ts);
  } while (rep<REPS);

}

