/*
 * Copyright (c) 2002 University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of SPAK.
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation is hereby granted without fee, provided that the
 * above copyright notice and this permission/disclaimer notice is
 * retained in all copies or modified versions, and that both notices
 * appear in supporting documentation.  THE COPYRIGHT HOLDERS PROVIDE
 * THIS SOFTWARE "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE COPYRIGHT
 * HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER
 * RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu
 * any improvements that they make and grant redistribution rights to the
 * University of Utah.
 *
 * Author: John Regehr (regehr@cs.utah.edu)
 */

#include "spak_public.h"

#define DBG_LEVEL 1

void avionics_test (void)
{ 
  struct task_set *ts1;
  double cs1;
  char fn[4096];

#define N_DIRS 4
#define N_WW 2
#define N_TASKS 18
#define max(x,y) ((x)>(y)?(x):(y))

  const char *dirs[] =  { "avionics1", "avionics2", "avionics3", "avionics4" };
  const char *ww[] = { "without", "with" };
  double best[N_WW][N_TASKS];
  struct task_set * bestts[N_WW][N_TASKS];
  int d, n, m;
  int mint; double mintcs; struct task_set *mintts;
  double maxcs; int maxcst; struct task_set *maxcsts;
  double u;

  max_resp = 1000000;

  for (m=0; m<N_WW; m++) {

    for (n=0; n<N_TASKS; n++) {
      best[m][n] = 0.0;
      bestts[m][n] = NULL;
    }
	
    printf ("** %s **\n", ww[m]);
	
    mint = 18; mintcs = 0.0;
    maxcs = 0.0; maxcst = 18;
    mintts = maxcsts = NULL;
	
    for (d=0; d<N_DIRS; d++) {
	  
      for (n=0; n<N_TASKS; n++) {
	sprintf (fn, "%s/task_set_%d_threads_%s_rtpt.txt", 
		 dirs[d], n, ww[m]);
	ts1 = load_task_set (fn);
	if (!ts1) continue;

	cs1 = find_critical_scale (ts1, NULL);
	u = utilization_set (ts1);

	if (cs1 > best[m][n]) {
	  best[m][n] = cs1;
	  bestts[m][n] = ts1;
	}

	if (n < mint || (n == mint && cs1 > mintcs)) {
	  mint = n;
	  mintcs = cs1;
	  mintts = ts1;
	}

	if (cs1 > maxcs) {
	  maxcs = cs1;
	  maxcst = n;
	  maxcsts = ts1;
	}

	/*
	  printf ("%d threads, %s run-time PT support, u = %f, cs = %f\n", 
	  n, ww[m], u, cs1);
	*/
      }
    }		

    for (n=0; n<N_TASKS; n++) {
      printf ("best schedule for %d threads has CS = %f\n",
	      n, best[m][n]);
      if (m==0 && (n==5 || n==6)) {
	optimal_partition_into_threads (bestts[m][n]);
	assert (feasible (bestts[m][n], TRUE) == 18);
	print_task_set (bestts[m][n]);
      }
    }

    printf ("mint = %d, cs = %f\n", mint, mintcs);
    printf ("maxcs = %f, maxcst = %d\n", maxcs, maxcst);

#if 0
    for (i=0; i<30; i++) {
      char x[512];
      // round to even percent just below where deadlines start to be missed
      double start = max (0.0, ((int)(100*(maxcs-1.01)))/100.0);
      printf ("start = %f\n", start);
      sprintf (x, "max_cs_%s", ww[m]);
      for (frac=start; frac <= 0.4;  frac+=0.01) {
	simulate (maxcsts, 2000 * MILLION, NULL, frac, x);
      }
      start = max (0.0, ((int)(100*(mintcs-1.01)))/100.0);
      sprintf (x, "min_t_%s", ww[m]);
      for (frac=start; frac <= 0.4;  frac+=0.01) {
	simulate (mintts, 2000 * MILLION, NULL, frac, x);
      }	
    }
#endif
  }
}

void find_hg_task_set (void)
{
  struct task_set *ts, *ts2;

  ts = create_task_set (3, 0, 0, 3,
			"feeb", 
			1000, 0, 0, 0,
			"Wang00_fixed");
  
  new_simple_task (ts,  3,  8,  8,  0, 0, "t0");
  new_simple_task (ts, 14, 33, 33,  8, 0, "t1");
  set_priorities (ts, DM);
  make_all_preemptible (ts);
  ts2 = copy_task_set (ts);
  
  feasible (ts, TRUE);
  print_task_set (ts);
  
  exit (0);

  // should be feasible with no jitter
  if (feasible (ts, TRUE) != num_tasks (ts)) goto end;
  
  // should be infeasible with jitter
  set_jitter (ts, 1, 8);
  if (feasible (ts, TRUE) == num_tasks (ts)) goto end;
  
  print_task_set (ts);
  
  set_jitter (ts, 1, 10);
  do {
    change_wcet (ts, 1, -1);
  } while (feasible (ts, TRUE) != num_tasks (ts));
  
  print_task_set (ts);
  
  assert (feasible (ts2, TRUE) == num_tasks (ts2));
  print_task_set (ts2);
  
  printf ("\n\n\n");
  
 end:
  free_task_set (ts);
  free_task_set (ts2);
}

