#!/usr/bin/perl -w

use strict;
use Statistics::Descriptive;

my $BUCKET_DIV = 10;
my $CON = (1.0/(2*$BUCKET_DIV));
my $FUDGE = ($CON/5);

my $num = 0;
my %pu = ();
my %po = ();
my %nu = ();
my %no = ();

my $pu_stat = Statistics::Descriptive::Full->new();
my $po_stat = Statistics::Descriptive::Full->new();
my $nu_stat = Statistics::Descriptive::Full->new();
my $no_stat = Statistics::Descriptive::Full->new();

while (my $line = <STDIN>) {

    if ($line =~ /preemptive CS: unoptimized ([0-9\.]+),  optimized ([0-9\.]+)$/) {

	$num++;

	my $u = $1;
	my $o = $2;

	my $bu = int ($u * $BUCKET_DIV);
	my $bo = int ($o * $BUCKET_DIV);
	
	if ($line =~ /non-preemptive/) {
	    $nu{$bu}++;
	    $nu_stat->add_data ($u);
	    $no{$bo}++;
	    $no_stat->add_data ($o);
	} else {
	    $pu{$bu}++;
	    $pu_stat->add_data ($u);
	    $po{$bo}++;
	    $po_stat->add_data ($o);
	}
    }
}

print "median non-preemptive unoptimized: ".($nu_stat->median())."\n";
print "median non-preemptive optimized  : ".($no_stat->median())."\n";
print "median preemptive unoptimized    : ".($pu_stat->median())."\n";
print "median non-preemptive optimized  : ".($po_stat->median())."\n";
print "\n";
print "mean non-preemptive unoptimized: ".($nu_stat->mean())."\n";
print "mean non-preemptive optimized  : ".($no_stat->mean())."\n";
print "mean preemptive unoptimized    : ".($pu_stat->mean())."\n";
print "mean non-preemptive optimized  : ".($po_stat->mean())."\n";
print "\n";
print "$num total data points\n";

sub bynum {
    return $a <=> $b;
}

open PU, ">pre_unopt.dat" or die ();
foreach my $key (sort bynum keys %pu) {
    print PU ($key/$BUCKET_DIV+($CON-$FUDGE))." ".($pu{$key})."\n";
}
close PU;

open PO, ">pre_opt.dat" or die ();
foreach my $key (sort bynum keys %po) {
    print PO ($key/$BUCKET_DIV+$CON+$FUDGE)." ".($po{$key})."\n";
}
close PO;

open NU, ">np_unopt.dat" or die ();
foreach my $key (sort bynum keys %nu) {
    print NU ($key/$BUCKET_DIV+($CON-$FUDGE))." ".($nu{$key})."\n";
}
close NU;

open NO, ">np_opt.dat" or die ();
foreach my $key (sort bynum keys %no) {
    print NO ($key/$BUCKET_DIV+$CON+$FUDGE)." ".($no{$key})."\n";
}
close NO;

