#!/usr/bin/perl -w

use strict;

my %thrd_map;
my %sens_map;
my $good;
my $bad;
my $current_ts;
my $total_dl;

sub init ()
{
    %thrd_map = ();
    %sens_map = ();
    $good = 0;
    $bad = 0;
    $current_ts = "";
    $total_dl = 0;
}

sub bynum {
    return $a <=> $b;
}

sub process_line ($)
{
    my $line = shift;
    chop $line;

    if ($line =~ /^task set (.+)$/) {
	$current_ts = $1;
	print "processing $current_ts\n";
	return;
    }

    if ($line =~ /^  for ([0-9]+) threads best cs is ([0-9\.]+)$/) {
	if ($2 != 0.0) {
	    $sens_map{$1} = $2;
	}
    }

    if ($line =~ /^([0-9]+) threads:  overrun by ([0-9\.]+) pct, ([0-9]+) missed deadlines \(total ([0-9]+)\)$/) {
	$total_dl = $4;
	# plot pct
	#$thrd_map{$1} .= ($2*100)." ".(100*$3/$4)."\n";
	# plot absolute number of misses
	$thrd_map{$1} .= ($2*100)." $3\n";
	$good++;
	return;
    }

    $bad++;
    
    print "$line\n";

    die if ($line =~ /missed/);
}

sub finish ()
{
    my $gpfile = "plot_$current_ts.gp";
    open OUTF2, ">$gpfile";
    print OUTF2 "set terminal postscript eps\n";
    print OUTF2 "set output \"plot_$current_ts.ps\"\n";
    print OUTF2 "set xlabel \"Percent Overrun\"\n";
    print OUTF2 "set ylabel \"Missed Deadlines (out of $total_dl)\"\n";
    print OUTF2 "set title \"$current_ts\"\n";
    print OUTF2 "plot [0:50] \\\n";
    
    my $first = 1;
    foreach my $key (sort bynum keys %thrd_map) {
	my $fn = $current_ts."_thrd_$key.dat";
	open OUTF, ">$fn";
	print OUTF $thrd_map{$key};
	close OUTF;
	if ($first) {
	    $first = 0;
	} else {
	    print OUTF2 ", ";
	}
	print OUTF2 "  \"$fn\" t \"$key threads; CS = $sens_map{$key}\" with lines \\\n";
    }
    
    print OUTF2 "\n";
    close OUTF2;
    
    system "gnuplot < $gpfile";

    print "used $good lines, ignored $bad.\n";
}

my $first = 1;

while (my $line = <STDIN>) {
    if ($line =~ /#################/) {
	if ($first) {
	    $first = 0;
	} else {
	    finish ();
	}
	init ();
    }
    process_line ($line);
}

finish ();
