#!/usr/bin/perl -w

use strict;
use Statistics::Distributions;

sub percentile ($$)
{
    my $pctile = (shift) / 100.0;
    my $x = shift;
    my @data = @$x;
    my $len = scalar (@data);
    return $data[int($len*$pctile)];
}

sub median ($) 
{
    return percentile (shift, 50);
}

sub mean ($)
{
    my $x = shift;
    my @data = @$x;
    my $total = 0.0;
    my $len = scalar (@data);

    foreach (@data) {
	$total += $_;
    }

    return $total / $len;
}

sub trim ($$)
{
    my $x = shift;
    my $frac = shift;
    my @data = @$x;
    my @data2 = @data;
    my $len = scalar (@data2);
    my $low = int($len*$frac);
    my $high = int($len-($len*$frac));
    my @data3 = @data2[$low..$high];
    return @data3;
}

sub stdev ($)
{
    my $x = shift;
    my @data = @$x;
    my $total;
    my $len = scalar (@data);

    my $avg = mean (\@data);

    $total = 0.0;
    foreach (@data) {
	my $inc = ($_ - $avg) * ($_ - $avg);
	#print "$_ $avg $inc\n";
	$total += $inc;
    }
    # $total /= $len-1;

    # FIXME
    $total /= $len;

    return sqrt ($total);
}

sub confidence ($$)
{
    my $x = shift;
    my $conf = shift;
    my @data = @$x;
    my $len = scalar (@data);
    my $avg = mean (\@data);
    my $a = 1.0-$conf;
    my $aa = 1.0-($a/2);
    my $z = Statistics::Distributions::udistr ($aa);
    my $std = stdev (\@data);
    print "  stdev = $std\n";
    # print "  z ($aa) = $z\n";
    my $low = $avg - ($z*$std/sqrt($len));
    my $high = $avg + ($z*$std/sqrt($len));
    return ($low, $high);
}

my %thrd_map;
my $good;
my $bad;
my $current_ts;
my $total_dl;

sub process_line ($)
{
    my $line = shift;
    chop $line;

    if ($line =~ /^([a-zA-Z\_]+) overrun by ([0-9\.]+) pct, ([0-9]+) missed deadlines \(total ([0-9]+)\)$/) {

	my $exp = $1;
	my $pct = $2*100;
	my $missed = $3;
	$total_dl = $4;

	if (($missed > 0) && ($exp =~ /without/)) {
	    push @{$thrd_map{$exp}{$pct}}, $missed;
	    $good++;
	}

	return;
    }

    $bad++;
    
    die if ($line =~ /missed/);
}

%thrd_map = ();
$good = 0;
$bad = 0;
$current_ts = "";
$total_dl = 0;

#open INF, "<avionics_misses.dat" or die ();
open INF, "<spak.out" or die ();

while (my $line = <INF>) {
    process_line ($line);
}

close INF;

sub bynum {
    return $a <=> $b;
}

my %strs;

my $first = 1;
foreach my $key (keys %thrd_map) {
    open OUTF2, ">$key.dat" or die;
    my $total = 0;
    foreach my $key2 (sort bynum keys %{$thrd_map{$key}}) {
	next if ($key2 > 30);
	my @l = @{$thrd_map{$key}{$key2}};
	my $mean = mean (\@l);
	print "  $key $key2 $mean\n";
	(my $low, my $high) = confidence (\@l, 0.90);
	$strs{$key} .= "  y_epts $key2 $mean $low $high\n";
    }
    close OUTF2;
    print "total of $total samples for $key\n";
}

print "used $good lines, ignored $bad.\n";

my %marks = (0 => "box",
	     1 => "x",
	     2 => "circle",
	     3 => "diamond");

my %labels = ("min_t_without" => "5 threads",
	      "max_cs_without" => "6 threads",
	      "min_t_with" => "foo",
	      "max_cs_with" => "bar");

open JGR, ">misses.jgr" or die;

print JGR "newgraph\n";
print JGR "  xaxis size 2.4 min 0 label : maximum percent overrun\n";
print JGR "  yaxis size 2.2 label : missed deadlines\n";

print JGR "  newline rarrows asize 0.4 25 pts 10.04 600\n 10.04 150\n";
print JGR "  newstring fontsize 8 x 10.04 y 650 : critical scaling factor \\\n for 5 threads = 1.104\n";

print JGR "  newline rarrows asize 0.4 25 pts 17.40 900\n  17.40 150\n";
print JGR "  newstring fontsize 8 x 17.40 y 950 : critical scaling factor \\\n for 6 threads = 1.174\n";

my $num = 0;
foreach (keys %strs) {
    print "$_\n";
    print JGR "newcurve marktype $marks{$num} label rotate 20 : $labels{$_}\n";
    print JGR $strs{$_};
    $num++;
}

print JGR "legend defaults x 10 y 1500\n";

close JGR;

system "jgraph < misses.jgr > misses.eps";
system "jgraph -P -L < misses.jgr > misses.ps";
