#!/usr/bin/perl -w

# converts SPAK simulator output into jgraph input, and runs jgraph
# to produce postscript

use strict;

if (scalar(@ARGV) != 3) {
    print "usage: render_sim_output.pl sim_file.txt start_time end_time\n\n";
    exit -1;
}

my $FN = $ARGV[0];
my $MIN = $ARGV[1];
my $MAX = $ARGV[2];

open INPUT, "<$FN" or die ("oops: can't open input file $FN\n");

my $data = "";
my $max_pos = 0;
my %pos = ();
my %label = ();
my $WIDTH = $MAX - $MIN;
my $CIRCLE_STR = "newcurve linetype solid marktype circle marksize ".(2.0/200*$WIDTH)." ";

sub insert ($$)
{
    my $name = shift;
    my $pri = shift;
    my $place;

    die if (defined($pos{$name}));

    printf "$name\n";

    $place = $pri;
    if ($place > $max_pos) {
	$max_pos = $place;
    }

    $pos{$name} = $place;
    $label{$place} = $name;
    if ($place > $max_pos) { $max_pos = $place; }
}

while (my $line = <INPUT>) {
    chomp $line;
    (my $f1, my $f2, my $f3, my $f4, my $f5) = split / /, $line;
    if ($f1 eq "run") {

	die if (!defined($f4) || defined ($f5));

	my $name = $f2;
	my $start = $f3;
	my $end = $f4;

	next if ($end < $MIN || $start > $MAX);
	if ($start < $MIN) { $start = $MIN; }
	if ($end > $MAX) { $end = $MAX; }

	die if (!defined ($pos{$name}));
	
        $data .= "newline poly pfill 0.8 pts\n";
	$data .= "  ".$start." ".$pos{$name}."\n";
	$data .= "  ".$start." ".($pos{$name}+0.5)."\n";
	$data .= "  ".$end." ".($pos{$name}+0.5)."\n";
	$data .= "  ".$end." ".$pos{$name}."\n";

    } elsif ($f1 eq "pri") {
	
	die if (!defined($f3) || defined ($f4));

	my $name = $f2;
	my $pri = $f3;

	insert ($name, $pri);

    } elsif ($f1 eq "release" || $f1 eq "completed" || 
	     $f1 eq "deadline" || $f1 eq "missed") {
	
	die if (!defined($f3) || defined($f4));

	my $cmd = $f1;
	my $name = $f2;
	my $time = $f3;

	next if ($time < $MIN || $time > $MAX);
	die if (!defined ($pos{$name}));

	if ($cmd eq "release") {
	    $data .= "$CIRCLE_STR fill 1.0 pts ";
	    $data .= $time." ".($pos{$name})."\n";
	} elsif ($cmd eq "deadline") {
	    $data .= "newline linetype solid ";
	    $data .= "pts ";
	    $data .= $time." ".($pos{$name}+0.7)." ".$time." ".($pos{$name})."\n";
	    $data .= "newline linetype solid ";
	    $data .= "pts ";
	    $data .= ($time-(1.2/200)*$WIDTH)." ".($pos{$name}-0.1)." ";
	    $data .= "$time ".$pos{$name}." ";
	    $data .= ($time+(1.2/200)*$WIDTH)." ".($pos{$name}-0.1)."\n";
	} elsif ($cmd eq "completed" || $cmd eq "missed") {
	    my $fill;
	    if ($cmd eq "completed") {
		$fill = "1.0";
	    } else {
		$fill = "0.0";
	    }
	    $data .= "$CIRCLE_STR fill $fill pts ";
	    $data .= $time." ".($pos{$name}+0.5)."\n";
	} else {
	    die;
	}

    } else {
	print "unknown directive $f1\n";
	die ();
    }
}

close INPUT;

my $header = "";
$header .= "newgraph\n";
$header .= "title : SPAK simulated execution trace\n";
$header .= "yaxis size 5.0 min 0.0 max ".($max_pos+1);
$header .= " gray 0.000000 hash 0.0 label fontsize 16 : Task (lower tasks have higher priority)\n";
$header .= "hash_labels fontsize 12\n";

for (my $i=0; $i<=$max_pos; $i++) {
    $header .= "hash_label at ".($i+0.4)." : ".$label{$i}."\n";
}

$header .= "xaxis size 9.0 min ".$MIN." max ".$MAX;
$header .= " gray 0 label fontsize 16 : Time\n";

open OUTPUT2, ">$FN.jgr";
print OUTPUT2 $header;
print OUTPUT2 $data;
close OUTPUT2;

system "jgraph < $FN.jgr > $FN.eps";
system "jgraph -P -L < $FN.jgr > $FN.ps";
